import findIndex from "lodash/findIndex";
import remove from "lodash/remove";
import { createAsyncAction, successResult } from "pullstate";

import { AnnouncementStore } from "stores";

/**
 * Add new announcement.
 */
export const addAnnouncement = createAsyncAction(
  async ({ content }) => {
    /** @type {CF2021.Announcement} */
    const payload = {
      id: "999",
      datetime: new Date(),
      type: "announcement",
      content,
      seen: true,
    };

    return successResult(payload);
  },
  {
    postActionHook: ({ result }) => {
      if (!result.error) {
        AnnouncementStore.update((state) => {
          state.items.push(result.payload);
        });
      }
    },
  }
);

/**
 * Delete existing announcement.
 */
export const deleteAnnouncement = createAsyncAction(
  /**
   *
   * @param {CF2021.Announcement} item
   */
  async (item) => {
    return successResult(item);
  },
  {
    postActionHook: ({ result }) => {
      if (!result.error) {
        AnnouncementStore.update((state) => {
          remove(state.items, { id: result.payload.id });
        });
      }
    },
  }
);

/**
 * Update content of an announcement.
 */
export const updateAnnouncementContent = createAsyncAction(
  /**
   *
   * @param {CF2021.Announcement} item
   * @param {string} newContent
   */
  async ({ item, newContent }) => {
    return successResult({ item, newContent });
  },
  {
    postActionHook: ({ result }) => {
      if (!result.error) {
        AnnouncementStore.update((state) => {
          const itemIdx = findIndex(state.items, {
            id: result.payload.item.id,
          });
          state.items[itemIdx].content = result.payload.newContent;
        });
      }
    },
  }
);
